Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.IO
Imports System.Windows.Media.Imaging
Imports System.Windows.Markup
Imports System.Threading
Imports System.Globalization

Namespace OpenFileDialogDemo
	Partial Public Class Page
		Inherits UserControl
		Private Const _cx As Integer = 400
		Private Const _cy As Integer = 300
		Private Const _border As Integer = 10
		Private Const _angle As Integer = 30
		Private _dragging As Boolean = False
		Private _lastx, _lasty As Double
		Private _owner As FrameworkElement
		Private _rand As New Random(DateTime.Now.Millisecond)
		Private _zindex As Integer = 100
		Private _count As Integer = 0
		Private _files As Queue(Of FileInfo) = New Queue(Of FileInfo)()

		Private Const _template As String = "<Border Name=""Photo_{0}"" Canvas.Left=""{1}"" Canvas.Top=""{2}"" Width=""{3}"" Height=""{4}"" Background=""White"" Visibility=""Collapsed"" xmlns=""http://schemas.microsoft.com/client/2007"">" & "<Border.RenderTransform>" & "<RotateTransform Name=""SpinTransform"" CenterX=""{5}"" CenterY=""{6}"" />" & "</Border.RenderTransform>" & "<Image Width=""{7}"" Height=""{8}"" Stretch=""UniformToFill"" /> " & "</Border>"

		Public Sub New()
			InitializeComponent()

			' Center the Open button horizontally
			Dim width As Double = Application.Current.Host.Content.ActualWidth
			OpenButton.SetValue(Canvas.LeftProperty, (width - OpenButton.Width) / 2.0)

			' Register a handler for the control's Resized events
			AddHandler Application.Current.Host.Content.Resized, AddressOf OnResized

			' Register a handler for Rendering events
			AddHandler CompositionTarget.Rendering, AddressOf CompositionTarget_Rendering
		End Sub

		Private Sub OpenButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Let the user browse for images
			Dim ofd As New OpenFileDialog()
			ofd.Filter = "JPEG Files (*.jpg;*.jpeg)|*.jpg;*.jpeg|PNG Files (*.png)|*.png|All Files (*.*)|*.*"
			ofd.FilterIndex = 1
			ofd.Multiselect = True

			If CBool(ofd.ShowDialog()) Then
				' Measure the width and height of the page and set the PhotoBoard canvas accordingly
				PhotoBoard.Width = Application.Current.Host.Content.ActualWidth
				PhotoBoard.Height = Application.Current.Host.Content.ActualHeight

				' Reset the PhotoBoard canvas and photo queue
				PhotoBoard.Children.Clear()
				_files.Clear()
				_count = 0

				' Place each FileInfo in a queue for processing in response to Rendering events
				For Each fi As FileInfo In ofd.Files
					_files.Enqueue(fi)
				Next fi
			End If
		End Sub

		Private Sub CompositionTarget_Rendering(ByVal sender As Object, ByVal e As EventArgs)
			If _files.Count <> 0 Then
				' Create a photo
				Dim fi As FileInfo = _files.Dequeue()
				CreatePhoto(fi)
			End If
		End Sub

		Private Sub CreatePhoto(ByVal fi As FileInfo)
			Using stream As Stream = fi.OpenRead()
				Dim bi As New BitmapImage()
				bi.SetSource(stream)

				' Create a Border containing an image
				Dim item As Border = CType(XamlReader.Load(String.Format(CultureInfo.InvariantCulture, _template, _count, (PhotoBoard.Width - _cx) / 2, (PhotoBoard.Height - _cy) / 2, _cx + _border, _cy + _border, (_cx + _border) / 2, (_cy + _border) / 2, _cx, _cy)), Border)
				_count += 1

				' Assign the image bits to the Image
				Dim image As Image = CType(item.Child, Image)
				image.Source = bi

				' Add the photo to the scene
				PhotoBoard.Children.Add(item)

				' Register event handlers
				AddHandler item.MouseLeftButtonDown, AddressOf OnButtonDown
				AddHandler item.MouseMove, AddressOf OnMouseMove
				AddHandler item.MouseLeftButtonUp, AddressOf OnButtonUp

				' Compute spatial parameters
				Dim x As Double = _rand.NextDouble() * (PhotoBoard.Width - _cx)
				Dim y As Double = _rand.NextDouble() * (PhotoBoard.Height - _cy)
				Dim angle As Double = _angle - (_rand.NextDouble() * _angle * 2)

				' Position the photo and make it visible
				item.SetValue(Canvas.LeftProperty, x)
				item.SetValue(Canvas.TopProperty, y)
				Dim transform As RotateTransform = CType(item.FindName("SpinTransform"), RotateTransform)
				transform.Angle = angle
				item.Visibility = Visibility.Visible
			End Using
		End Sub

		Private Sub OnButtonDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
			' Move the photo to the top of the Z-order
			CType(sender, FrameworkElement).SetValue(Canvas.ZIndexProperty, _zindex)
			_zindex += 1

			' Record the current mouse position
			_lastx = e.GetPosition(Nothing).X
			_lasty = e.GetPosition(Nothing).Y

			' Begin dragging
			_owner = CType(sender, FrameworkElement)
			CType(sender, FrameworkElement).CaptureMouse()
			_dragging = True
		End Sub

		Private Overloads Sub OnMouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
			If _dragging Then
				Dim x As Double = e.GetPosition(Nothing).X
				Dim y As Double = e.GetPosition(Nothing).Y

				Dim dx As Double = x - _lastx
				Dim dy As Double = y - _lasty

				_lastx = x
				_lasty = y

				CType(sender, FrameworkElement).SetValue(Canvas.LeftProperty, CDbl((CType(sender, FrameworkElement)).GetValue(Canvas.LeftProperty)) + dx)
				CType(sender, FrameworkElement).SetValue(Canvas.TopProperty, CDbl((CType(sender, FrameworkElement)).GetValue(Canvas.TopProperty)) + dy)
			End If
		End Sub

		Private Sub OnButtonUp(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
			CType(sender, FrameworkElement).ReleaseMouseCapture()
			_dragging = False
		End Sub

		Private Sub OnResized(ByVal sender As Object, ByVal e As EventArgs)
			' When the control resizes, center the Open button horizontally
			Dim width As Double = Application.Current.Host.Content.ActualWidth
			OpenButton.SetValue(Canvas.LeftProperty, (width - OpenButton.Width) / 2.0)
		End Sub
	End Class
End Namespace
